<?php

namespace Bixo\Lead\Http\Controllers;

use Exception;
use Bixo\Lead\Actions\BlacklistWorkflow;
use Bixo\Lead\Http\Requests\BlacklistWorkflowRequest;
use Bixo\Lead\Http\Resources\BlacklistResource;
use Bixo\Lead\Models\Blacklist;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class BlacklistWorkflowController extends BaseController
{
    /**
     * Action controller function for blacklist.
     *
     * @param Model $blacklist
     * @param action next action for the blacklist.
     *
     * @return Response
     */
    public function __invoke(BlacklistWorkflowRequest $request, Blacklist $blacklist, $trasition)
    {
        try {
            $request = $request->all();
            $blacklist = BlacklistWorkflow::run($trasition, $blacklist, $request);
            $data = new BlacklistResource($blacklist);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('lead::blacklist.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('lead/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('lead/blacklist/' . $blacklist->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the blacklist and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $blacklist
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $blacklist, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $blacklist = Blacklist::findBySignedId($blacklist);
        $transitions = $this->transitions($blacklist, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('lead::blacklist.name') . ' ' . trans('Approvals'))
            ->view('lead::public.blacklist.workflow')
            ->layout('mobile')
            ->data(compact('blacklist', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the blacklist.
     *
     * @param PublicRequest $request
     * @param string $blacklist
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $blacklist, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $blacklist = Blacklist::findBySignedId($blacklist);
        $user = user();
        $blacklist = BlacklistWorkflow::run($transition, $blacklist, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($blacklist, $user)
    {
        $transitions = [];
        foreach ($blacklist->workflow()->transitions($blacklist) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.blacklist.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'blacklist' => $blacklist->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $blacklist->workflow()->form($value);
            $array['label'] = trans('lead::blacklist.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
